@@ -155,18 +155,19 @@ avatar_image_add_filter (EmpathyAvatarImage *avatar_image)
 	mask = PropertyChangeMask;
 
 	window = GDK_ROOT_WINDOW ();
-	gdkwindow = gdk_xid_table_lookup (window);
+	gdkwindow = gdk_x11_window_lookup_for_display (gdk_display_get_default (),
+		window);
 
 	gdk_error_trap_push ();
 	if (gdkwindow) {
 		XWindowAttributes attrs;
-		XGetWindowAttributes (gdk_display, window, &attrs);
+		XGetWindowAttributes (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), window, &attrs);
 		mask |= attrs.your_event_mask;
 	}
 
-	XSelectInput (gdk_display, window, mask);
+	XSelectInput (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), window, mask);
 
-	gdk_error_trap_pop ();
+	gdk_error_trap_pop_ignored ();
 
 	gdk_window_add_filter (NULL, avatar_image_filter_func, avatar_image);
 }
