@@ -145,6 +145,9 @@ void        empathy_send_file_from_uri_list             (EmpathyContact   *conta
 void        empathy_send_file_with_file_chooser         (EmpathyContact   *contact);
 void        empathy_receive_file_with_file_chooser      (EmpathyFTHandler *handler);
 
+/* Misc */
+void        empathy_make_color_whiter                   (GdkRGBA *color);
+
 G_END_DECLS
 
 #endif /*  __EMPATHY_UI_UTILS_H__ */
