@@ -54,6 +54,7 @@
 static gboolean only_if_needed = FALSE;
 static gboolean hidden = FALSE;
 static gchar *selected_account_name = NULL;
+static gboolean account_manager_prepared = FALSE;
 
 static void
 account_prepare_cb (GObject *source_object,
@@ -92,7 +93,6 @@ account_manager_ready_for_accounts_cb (GObject *source_object,
     gpointer user_data)
 {
   TpAccountManager *manager = TP_ACCOUNT_MANAGER (source_object);
-  const gchar *account_id = (const gchar*) user_data;
   GError *error = NULL;
 
   if (!tp_account_manager_prepare_finish (manager, result, &error))
@@ -102,7 +102,9 @@ account_manager_ready_for_accounts_cb (GObject *source_object,
       return;
     }
 
-  if (account_id != NULL)
+  account_manager_prepared = TRUE;
+
+  if (selected_account_name != NULL)
     {
       gchar *account_path;
       TpAccount *account = NULL;
@@ -111,7 +113,7 @@ account_manager_ready_for_accounts_cb (GObject *source_object,
       /* create and prep the corresponding TpAccount so it's fully ready by the
        * time we try to select it in the accounts dialog */
       account_path = g_strdup_printf ("%s%s", TP_ACCOUNT_OBJECT_PATH_BASE,
-          account_id);
+          selected_account_name);
       bus = tp_dbus_daemon_dup (NULL);
       if ((account = tp_account_new (bus, account_path, &error)))
         {
@@ -123,6 +125,8 @@ account_manager_ready_for_accounts_cb (GObject *source_object,
           DEBUG ("Failed to find account with path %s: %s", account_path,
               error->message);
           g_clear_error (&error);
+
+          maybe_show_accounts_ui (manager);
         }
 
       g_object_unref (bus);
@@ -134,27 +138,38 @@ account_manager_ready_for_accounts_cb (GObject *source_object,
     }
 }
 
-static void
-app_activated_cb (GtkApplication *app)
+static int
+app_command_line_cb (GApplication *app,
+    GApplicationCommandLine *cmdline)
 {
-  TpAccountManager *account_manager;
+  g_application_hold (app);
 
-  account_manager = tp_account_manager_dup ();
+  /* if the window is ready, present it; otherwise, it will be presented when
+   * the accounts manager is prepared */
+  if (account_manager_prepared)
+    {
+      TpAccountManager *account_manager;
 
-  empathy_accounts_show_accounts_ui (account_manager, NULL,
-          G_CALLBACK (gtk_main_quit));
+      account_manager = tp_account_manager_dup ();
+      empathy_accounts_show_accounts_ui (account_manager, NULL,
+              G_CALLBACK (gtk_main_quit));
 
-  g_object_unref (account_manager);
+      g_object_unref (account_manager);
+    }
+
+  return 0;
 }
 
-#define COMMAND_ACCOUNTS_DIALOG 1
-
-int
-main (int argc, char *argv[])
+static gboolean
+local_cmdline (GApplication *app,
+    gchar ***arguments,
+    gint *exit_status)
 {
-  TpAccountManager *account_manager;
+  gint i;
+  gchar **argv;
+  gint argc = 0;
+  gboolean retval = FALSE;
   GError *error = NULL;
-  GtkApplication *app;
 
   GOptionContext *optcontext;
   GOptionEntry options[] = {
@@ -164,7 +179,7 @@ main (int argc, char *argv[])
         NULL },
       { "if-needed", 'n',
         0, G_OPTION_ARG_NONE, &only_if_needed,
-        N_("Don't display any dialogs if there are any non-Salut accounts"),
+        N_("Don't display any dialogs unless there are only \"People Nearby\" accounts"),
         NULL },
       { "select-account", 's',
         G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_STRING, &selected_account_name,
@@ -175,23 +190,43 @@ main (int argc, char *argv[])
       { NULL }
   };
 
-  g_thread_init (NULL);
-  empathy_init ();
-
   optcontext = g_option_context_new (N_("- Empathy Accounts"));
   g_option_context_add_group (optcontext, gtk_get_option_group (TRUE));
   g_option_context_add_main_entries (optcontext, options, GETTEXT_PACKAGE);
 
+  argv = *arguments;
+  for (i = 0; argv[i] != NULL; i++)
+    argc++;
+
   if (!g_option_context_parse (optcontext, &argc, &argv, &error))
     {
       g_print ("%s\nRun '%s --help' to see a full list of available command line options.\n",
           error->message, argv[0]);
       g_warning ("Error in empathy init: %s", error->message);
-      return EXIT_FAILURE;
+
+      *exit_status = EXIT_FAILURE;
+      retval = TRUE;
     }
 
   g_option_context_free (optcontext);
 
+  return retval;
+}
+
+#define COMMAND_ACCOUNTS_DIALOG 1
+
+int
+main (int argc, char *argv[])
+{
+  TpAccountManager *account_manager;
+  GtkApplication *app;
+  GObjectClass *app_class;
+  gint retval;
+
+  g_thread_init (NULL);
+  empathy_init ();
+
+  gtk_init (&argc, &argv);
   empathy_gtk_init ();
 
   g_set_application_name (_("Empathy Accounts"));
@@ -199,20 +234,23 @@ main (int argc, char *argv[])
   gtk_window_set_default_icon_name ("empathy");
   textdomain (GETTEXT_PACKAGE);
 
-  app = gtk_application_new (EMPATHY_ACCOUNTS_DBUS_NAME, &argc, &argv);
+  app = gtk_application_new (EMPATHY_ACCOUNTS_DBUS_NAME,
+      G_APPLICATION_HANDLES_COMMAND_LINE);
+  app_class = G_OBJECT_GET_CLASS (app);
+  G_APPLICATION_CLASS (app_class)->local_command_line = local_cmdline;
 
   account_manager = tp_account_manager_dup ();
 
   tp_account_manager_prepare_async (account_manager, NULL,
-    account_manager_ready_for_accounts_cb, selected_account_name);
+    account_manager_ready_for_accounts_cb, NULL);
 
-  g_signal_connect (app, "activated",
-      G_CALLBACK (app_activated_cb), NULL);
+  g_signal_connect (app, "command-line", G_CALLBACK (app_command_line_cb),
+      NULL);
 
-  gtk_application_run (app);
+  retval = g_application_run (G_APPLICATION (app), argc, argv);
 
   g_object_unref (account_manager);
   g_object_unref (app);
 
-  return EXIT_SUCCESS;
+  return retval;
 }
